import QtQuick 1.0

/*
    Represent an image with an optional loading image and a default image to
    show when the source is empty. If source is undefined, the image is not
    shown at all.

    We have three cases:

    1) `source` is undefined (this can be the case for example in the user
       history list): we don't show anything, and the size is set to 0.

    2) `source` is set to "" or the empty url: we show the default image.

    3) `source` is set to a valid url: we show the url.
*/
Item {
    id: root
    property variant source
    property url loadingSource: defaultSource
    property url defaultSource
    property url background

    // If set force the images to a given size.
    property variant imageWidth
    property variant imageHeight

    // Set to true to rotate the loading image
    property bool loadingRotate: false

    // We use the global scale by default.
    // XXX: when we get already scaled resources, then we can use 1 as the
    // default value here.
    property real scale: rootWindow.style.scale

    width: (source !== undefined)? content.width : 0
    height: (source !== undefined)? content.height: 0
    opacity: (source !== undefined)? 1 : 0

    // Private properties
    // Used to get the source as a string.
    property string _sourceStr: root.source ? root.source : ""
    property int _width: (imageWidth ? imageWidth : Math.max(image.sourceSize.width, loadingImage.sourceSize.width)) * scale
    property int _height: (imageHeight ? imageHeight : Math.max(image.sourceSize.height, loadingImage.sourceSize.height)) * scale

    Item {
        id: content
        width: Math.max(backgroundImage.width, _width)
        height: Math.max(backgroundImage.height, _height)
        Image {
            id: backgroundImage
            source: root.background
            width: sourceSize.width * root.scale
            height: sourceSize.height * root.scale
            anchors.centerIn: parent
        }

        Image {
            id: errorImage
            anchors.centerIn: parent
            smooth: true
            width: _width
            height: _height
            source: root.defaultSource
            opacity: (image.status === Image.Error)? 1 : 0
        }

        Image {
            id: image
            anchors.centerIn: parent
            smooth: true
            width: _width
            height: _height
            source: (root._sourceStr)? root._sourceStr : root.defaultSource
        }

        Image {
            id: loadingImage
            anchors.centerIn: parent
            source: loadingSource
            smooth: true
            opacity: (image.status === Image.Loading)? 1 : 0
            width: _width
            height: _height

            NumberAnimation on rotation {
                from: 360; to: 0
                duration: 1000
                running: loadingRotate && loadingImage.visible
                loops: Animation.Infinite
            }
        }
    }
}
