import QtQuick 1.0
import Qt.labs.components.native 1.0

import "Network.js" as Network
import "Utils.js" as Utils


// TODO: fix support for the 'page' attribute
ListView {
    id: list

    property variant requestParams
    property string requestUrl
    property int page: 1
    // If set to true, we increment the page count every time we request more items from the server.
    // If set to false (default), we use the max_id argument to tell the server where to start from.
    property bool usePages: false
    // The http parameter used to set the number of items.
    property string countParam: "count"
    // How many items do we fetch the first time
    property int initialCount: 10
    // How many items do we fetch on 'more'
    property int moreCount: 10
    property bool autoRefresh: true

    // Set this to show an error message instead of the list.
    property variant errorMessage

    property bool loading: false
    property bool showLoading: loading

    // If set define the max number of item in the list.
    // This is used to decide when to show or not the more button.
    property variant maxCount
    property string headerText

    signal itemClicked(string id)

    // Private properties.
    property int count: initialCount
    property bool canGetMore: (maxCount === undefined) || (model.count < maxCount)
    property string type: "List" // This is used to find all the children lists of a page

    model: ListModel {id: model }
    clip: true
    cacheBuffer: 1000

    header: Rectangle {
        id: headerItem
        property string text: headerItem.ListView.view.headerText
        opacity: text ? 1 : 0
        width: text? parent.width : 0
        height: text ? childrenRect.height + 10 : 0
        color: "#dbdbdb"
        Text {
            y: 5
            anchors {
                left: parent.left
                leftMargin: 10;
            }
            text: headerItem.text
            font.pixelSize: rootWindow.style.fontNormalSize
        }
    }

    // XXX: For the moment the more button is entirely made here.
    // We should use ImageButton instead.  All the buttons should use
    // BorderImage.

    // TODO: Is there a way to clean up all those ugly "footerItem.ListView.view"?
    footer: Rectangle {
        id: footerItem
        anchors {
            left: parent.left
            right: parent.right
        }
        height: (moreText.text)? 50 * rootWindow.style.scale : 0
        opacity: (moreText.text)? 1 : 0
        color: (internal.state === "Pressed")? "grey" : "lightgrey"

        Row {
            anchors.horizontalCenter: parent.horizontalCenter
            height: parent.height
            spacing: 7
            BusyIndicator {
                visible: footerItem.ListView.view.showLoading
                anchors.verticalCenter: parent.verticalCenter
                height: 30
                width: 30
                running: true
            }
            Text {
                id: moreText
                anchors.verticalCenter: parent.verticalCenter
                font.pixelSize: rootWindow.style.fontNormalSize
                text: {
                    if (footerItem.ListView.view.showLoading)
                        return qsTr("Loading...");
                    if (footerItem.ListView.view.errorMessage)
                        return footerItem.ListView.view.errorMessage;
                    if (footerItem.ListView.view.model.count === 0)
                        return qsTr("No items");
                    if (footerItem.ListView.view.canGetMore)
                        return qsTr("More");
                    return "";
                }
            }
        }
        StateGroup {
            id: internal
            states: [
                State { name: "Pressed" },
                State { name: "" }
            ]
        }
        MouseArea {
            anchors.fill: parent
            // XXX: clean that mess!
            enabled: !footerItem.ListView.view.showLoading && !footerItem.ListView.view.errorMessage && footerItem.ListView.view.canGetMore

            onClicked: {
                footerItem.ListView.view.onMore()
            }

            onPressed: {
                internal.state = "Pressed"
            }
            onReleased: {
                internal.state = ""
            }

            onCanceled: {
                internal.state = ""
            }
        }
    }

    section.delegate: Rectangle {
        id: listHeading
        width: (parent)? parent.width : 0
        height: childrenRect.height + 10

        color: "#dbdbdb"
        Text {
            y: 5
            anchors {
                left: parent.left
                leftMargin: 10;
            }
            text: section
            font.pixelSize: rootWindow.style.fontNormalSize
        }
    }

    function refresh() {
        // There seems to be a bug in qml, if we use clear as we should,
        // some strange things happens when we try to repopulate the model.
        // So the hack here is to remove the elements one by one!

        // model.clear().

        // If we are already doing a request we don't refresh.
        // TODO: instead we should abort the current request and start a new
        // one.  Currently it is impossible to assign XMLHttpRequest object
        // to properties, so the only way to do that would be to store a map
        // off all the requests in a js file.
        if (loading === true)
            return;
        // This line is here to solve the bug that the loading button doesn't
        // show when we are not refreshing with the position at the end of the
        // list.  No idea why!
        positionViewAtIndex(model.count - 1, ListView.End);
        while (model.count > 0)
            model.remove(0);
        page = 1;
        count = initialCount;
        sendRequest();
    }

    // Reimplement if the id is not the in the 'id' attribute of the models.
    function getId(model) {
        return model.id
    }

    // A little bit ugly.
    function onMore() {
        // Adjust the page if we change the count.  For example if we had:
        // page = 1 and count = 20, if the next request has count = 10, then
        // page should be 3 (instead of 2).
        if (count === initialCount)
            page += (initialCount - moreCount) / moreCount
        count = moreCount
        if (usePages) {
            page += 1
            sendRequest()
        } else {
            var maxId = getId(model.get(model.count - 1))
            sendRequest(maxId)
        }
    }

    function sendRequest(maxId) {
        for (var i in requestParams) {
            if (requestParams[i] === undefined)
                return
        }

        loading = true
        var params = requestParams
        if (maxId) {
            params = Utils.extend({"max_id": maxId}, params)
        } else {
            params = Utils.extend({"page": page}, params)
        }
        if (countParam)
            params[countParam] = count

        var onLoaded = function(status, obj) {
            if (status == 200) {
                try {
                    parseResult(obj, model);
                } catch (err) {
                    console.log("Error: %1:%2: %3".arg(err.fileName).arg(err.lineNumber).arg(err.message));
                    errorMessage = qsTr("Internal error");
                }
            } else {
                errorMessage = qsTr("Connection error");
            }
            loading = false;
        }

        Network.doPost(requestUrl, params, onLoaded)
    }

    // When the request parameter change, we automatically refresh the list.
    onRequestParamsChanged: {
        // We shouldn't do it when maxCount === 1, but the problem is that we might call this before
        // maxCount has been set.  The best way is maybe to use a better trigger than
        // onRequestParamsChanged.
        if (autoRefresh)
            refresh();
    }
}


