import QtQuick 1.0
import Qt.labs.components.native 1.0
import JiePang 1.0 as JiePang

import "Common" as Common
import "Common/Storage.js" as Storage
import "Common/Network.js" as Network

Window {
    id: rootWindow

    property Item user: Common.User {}
    property Item mainPage

    // Disable all interactions when the pages are moving.
    enabled: !pageStack.busy

    // This contains some globals properties that should be used for the
    // items style.
    property Item style : Item {
        property int fontNormalSize: 24
        property int fontSmallSize: 20
        property int fontLargeSize: 32
        property real scale:  4./3.
    }

    // The global position manager object
    property Item positionManager: Common.PositionManager {
        onLongitudeChanged: {
            print("Got new longitude", longitude)
        }

        onLatitudeChanged: {
            print("Got new latitude", latitude)
        }
    }

    JiePang.NdefManager {
        onReadJiePangRecord: {
            print("Got jiepang NFC record", type, uid);
            var url = "/locations/show.json"
            var args = {nfc: uid}
            var onRet = function(ret, obj) {
                var currentPage = pageStack.currentPage;
                if (currentPage.pageName === "CheckinPage") {
                    if (currentPage.guid === obj.guid)
                        return;
                    pageStack.pop(undefined, true);
                }

                pushPage(Qt.resolvedUrl("Pages/CheckInPage.qml"),
                         {"guid": obj.guid,
                             "longitude": obj.lon,
                             "latitude": obj.lat,
                             "nfcKey": obj.nfc_key});
            }

            rootWindow.request(url, args, undefined, undefined, onRet);
        }
    }

    JiePang.DeviceInfo {
        id: deviceInfo
    }

    Rectangle {
        anchors.fill: parent
        color: "white"
    }

    PageStack {
        id: pageStack
        toolBar: toolBar
        anchors {
            left: parent.left
            right: parent.right
            bottom: toolBar.top
            bottomMargin: 0
            top: parent.top
            topMargin: 0
        }
    }

    ToolBar {
        id: toolBar
        anchors.bottom: parent.bottom
        style: ToolBarStyle {
            inverted: true
        }
    }

    function popPage() {
        pageStack.pop()
    }

    function pushPage(url, attrs, immediate) {
        return pageStack.push(url, attrs, immediate)
    }

    function showPlacesTab() {
        pageStack.pop(null) // Return to the main page.
        pageStack.currentPage.showPlacesTab()
    }

    function refreshFeeds() {
        mainPage.refreshFeeds();
    }
    Component {
        id: queryDialogComponent
        Common.QueryDialog {
            id: rootDialog
        }
    }

    Component.onCompleted: {
        Storage.initialize()
        var firstRun = Storage.getSetting("first", "")
        console.debug("first",firstRun)
        if (firstRun !== "1") {
            var d_dialog = queryDialogComponent.createObject(pageStack)
            d_dialog.acceptButtonText = qsTr("Confirm")
            d_dialog.rejectButtonText = qsTr("Quit")
            d_dialog.titleText = qsTr("Disclaimer")
            d_dialog.message = qsTr("You are about to use a third party application that is not owned or developed by Nokia.Nokia shall have no liability for any aspect of the application whatsoever, including but not limited to its performance, all cost incurred during usage, intellectual property rights or support. Please check the corresponding Application Guide for full details. \n\nBy clicking 'OK' below you acknowledge that you have read and understand, and that you agree to be bound by the terms above and in the Application Guide. \n\nThis application is created and designed by Wozai Info Tech(Beijing) Co.,Ltd. \n\n\nJiepang Service Email: support@jiepang.zendesk.com \n\nTel: +86-010-65817399")
            d_dialog.accepted.connect(function() {console.debug("accepted");Storage.setSetting("first","1")})
            d_dialog.rejected.connect(function() {console.debug("rejected");Qt.quit()})
            d_dialog.open()
        }
//        checkNewVersion();

        mainPage = pageStack.push(Qt.resolvedUrl("Pages/MainPage.qml"))
        var login = Storage.getSetting("login")
        var password = Storage.getSetting("password")

        if (login && password) {
            signIn(login, password, true, function(result) {
                       if (!result)
                           pageStack.push(Qt.resolvedUrl("Pages/LoginPage.qml"),
                                          {"login": login, "password": password})
                       return
                   })
        } else {
            pageStack.push(Qt.resolvedUrl("Pages/LoginPage.qml"))
        }
    }

    // Check if there is new version available and if so propose the user
    // to download it.
    // if `foreground` is set to true, show a waiting dialog and also tell
    // the user if no update is available.
    function checkNewVersion(foreground) {
        var url = "/account/check_symbian.json";

        // Generate the version number from the version string.
        var pad2 = function(x) {return (x.length == 1) ? "0" + x : x}
        var ver_numbers = version.split('.');
        var var_index = parseInt(pad2(ver_numbers[0]) +
                                 pad2(ver_numbers[1]) +
                                 pad2(ver_numbers[2]), 10);
        var args = {"ver": var_index};
        console.debug("ver",var_index)
        var onResult = function(ret, obj) {
            if (obj.uri) { // New version available
                var dialogComponent = Qt.createComponent("Pages/NewVersionDialog.qml");
                var dialog = dialogComponent.createObject(
                        pageStack,
                        {"message": obj.message, "uri": obj.uri});
                dialog.open();
            } else if (foreground) {
                showDialog(qsTr("Update"), qsTr("No update available"));
            }
        }
        var msg = foreground ? undefined : null
        rootWindow.request(url, args, msg, undefined, onResult);
    }

    function showWaitDialog(msg) {
        if (msg === undefined)
            msg = qsTr("Loading...");
        var dialog = waitDialogComponent.createObject(mainPage);
        dialog.titleText = msg;
        dialog.open();
        return dialog;
    }

    function showDialog(title, msg) {
        var dialog = dialogComponent.createObject(mainPage);
        dialog.titleText = title;
        dialog.message = msg;
        dialog.open();
        return dialog;
    }

    // Make a server request.
    //
    // Params:
    // - url: url of the request, if only the path is specified, then the
    //   default server address is used.
    // - params: dict of the params to send with the request.
    // - msg: message to show while we wait for the answer.  Set to null for
    //   no message.
    // - errMsg: title of the error message in case of error.
    // - callback: called in case of success.
    // - errCallback: called in case of error.
    function request(url, params, msg, errMsg, callback, errCallback) {
        // Set default values
        if (msg === undefined)
            msg = qsTr("Waiting");
        if (!errMsg)
            errMsg = qsTr("Error");
        if (!errCallback)
            errCallback = function() {}

        if (msg)
            var waitDialog = showWaitDialog(msg);
        Network.doPost(url, params, function(ret, obj) {
                           if (msg)
                               waitDialog.close();
                           if (ret !== 200) {
                               var message;
                               if (obj.error && obj.error.message)
                                   message = obj.error.message;
                               else
                                   message = qsTr("Unknown error");

                               var dialog = showDialog(errMsg, message);
                               dialog.accepted.connect(function() {errCallback(ret, obj)});
                           } else {
                               callback(ret, obj);
                           }
                       });
    }

    Component {
        id: dialogComponent
        QueryDialog {
            id: dialog
            acceptButtonText: qsTr("Ok")
        }
    }

    Component {
        id: waitDialogComponent
        Common.Popup {
            property string titleText: qsTr("waiting")
            Row {
                anchors.horizontalCenter: parent.horizontalCenter
                spacing: 10
                BusyIndicator {
                    height: 60
                    width: 60
                    running: true
                    style: BusyIndicatorStyle {
                        inverted: true
                    }
                }
                Text {
                    text: titleText
                    color: "white"
                    font.pixelSize: rootWindow.style.fontNormalSize
                }
            }
        }
    }

    function signIn(login, password, saveAccount, callback) {
        var dialog = showWaitDialog(qsTr("Signing In"));
        Network.setCredentials(login, password);
        // Set the source parameter for all our server calls
        var source = deviceInfo.manufacturer + " " + deviceInfo.model;
        if (deviceInfo.manufacturer === "Simulator Manufacturer")
            source = "nokia simulator";
        Network.setSource(source);
        var onResult = function(status, obj) {
            print("TEST close wait dialog!");
            dialog.close();
            if (status == 200) {
                if (saveAccount) {
                    Storage.setSetting("login", login)
                    Storage.setSetting("password", password)
                } else {
                    Storage.setSetting("login", "")
                    Storage.setSetting("password", "")
                }

                user.userId = obj.id
                user.nick = obj.nick
                user.city = obj.city
                positionManager.refresh() // TODO: we should maybe use a signal instead
                callback(true)
                return
            } else if (status == 401) { // wrong credentials
                Storage.setSetting("login", "")
                Storage.setSetting("password", "")
                showDialog(qsTr("Cannot sign-in"), qsTr("Wrong Login"));
            } else if (status === 0) {
                showDialog(qsTr("Cannot sign-in"), qsTr("Server connection error, try again later."));
            }

            callback(false)
        }

        Network.doPost("/account/verify_credentials.json", null, onResult)
    }

    function signOut() {
        Storage.setSetting("login", "")
        Storage.setSetting("password", "")
        pageStack.pop(null, true)
        pageStack.push(Qt.resolvedUrl("Pages/LoginPage.qml"))
    }

}

